#region Copyright (c) 2002-2005, Bas Geertsema, Xih Solutions (http://www.xihsolutions.net)
/*
Copyright (c) 2002-2005, Bas Geertsema, Xih Solutions (http://www.xihsolutions.net)
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright 
notice, this list of conditions and the following disclaimer in the 
documentation and/or other materials provided with the distribution.
* Neither the names of Bas Geertsema or Xih Solutions nor the names of its 
contributors may be used to endorse or promote products derived 
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE. */
#endregion

using System;
using System.Runtime.Serialization;
using XihSolutions.DotMSN.Core;
using XihSolutions.DotMSN.DataTransfer;

namespace XihSolutions.DotMSN
{
	/// <summary>
	/// Indicates an authentication error with the Nexus service.
	/// </summary>
	/// <remarks>
	/// This exception is thrown when the Nexus service gives back a 401 Unauthorized header. This occurs most likely due to an invalid account or password.
	/// </remarks>
	[Serializable()]
	public class UnauthorizedException : DotMSNException
	{
		/// <summary>
		/// Constructor.
		/// </summary>
		public UnauthorizedException()
		{
		}

		/// <summary>
		/// Specifies a general exception.
		/// </summary>
		/// <param name="message">A textual presentation of the exception message</param>
		public UnauthorizedException(string message)
			: base(message)
		{		
		}
		
		/// <summary>
		/// Specifies a general exception but which originates from another exception.
		/// </summary>
		/// <param name="message">A textual presentation of the exception message</param>
		/// <param name="innerException">The (inner)exception which caused this exception. For example a SocketException.</param>
		public UnauthorizedException(string message, Exception innerException)
			: base(message, innerException)
		{
		}

		/// <summary>
		/// Serialization constructor.
		/// </summary>
		/// <param name="info"></param>
		/// <param name="context"></param>
		protected UnauthorizedException(SerializationInfo info, StreamingContext context) : base(info, context)
		{
		
		}
	}

}
